<?php

/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

use admin\BayarcashPortalApi;

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/controllers/admin/BayarcashPortalApi.php';
require_once dirname(__FILE__) . '/controllers/admin/BayarcashConfigForm.php';

class Bayarcash extends PaymentModule
{
    public const CONFIG_ORDER_STATE = 'BAYARCASH_ORDER_STATE';
    // Configuration keys
    public const CONFIG_ENABLED = 'BAYARCASH_ENABLED';
    public const CONFIG_PAT = 'BAYARCASH_PAT';
    public const CONFIG_API_KEY = 'BAYARCASH_API_KEY';
    public const CONFIG_SECRET_KEY = 'BAYARCASH_SECRET_KEY';
    public const CONFIG_TEST_MODE = 'BAYARCASH_TEST_MODE';
    public const CONFIG_PAYMENT_CHANNELS = 'BAYARCASH_PAYMENT_CHANNELS';

    private const MODULE_ADMIN_CONTROLLER = 'AdminBayarcashConfiguration';

    // Required hooks for module functionality
    private const HOOKS = [
        'paymentOptions',
        'displayAdminOrderMainBottom',
        'displayHeader',
        'displayBackOfficeHeader',
        'displayPaymentReturn',
    ];

    /** @var BayarcashPortalApi */
    private $portalApi;

    public function __construct()
    {
        $this->setupModuleBasics();
        $this->setupPaths();
        $this->setupPaymentSettings();

        // Initialize portal API
        $this->portalApi = new BayarcashPortalApi(
            (bool)Configuration::get(self::CONFIG_TEST_MODE),
            Configuration::get(self::CONFIG_PAT)
        );

        parent::__construct();

        $this->setupDisplayInfo();
        $this->checkRequirements();
    }

    private function setupModuleBasics(): void
    {
        $this->name = 'bayarcash';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'Bayarcash Sdn Bhd';
        $this->need_instance = 1;
        $this->bootstrap = true;
    }

    private function setupPaths(): void
    {
        if (!$this->_path) {
            $this->_path = __PS_BASE_URI__ . 'modules/' . $this->name . '/';
        }
        $this->js_path = $this->_path . 'views/js/';
        $this->css_path = $this->_path . 'views/css/';
    }

    private function setupPaymentSettings(): void
    {
        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => _PS_VERSION_,
        ];
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->controllers = ['validation', 'callback'];
    }

    private function setupDisplayInfo(): void
    {
        $this->displayName = $this->l('Bayarcash');
        $this->description = $this->l('Accept payment from Malaysia. Bayarcash support FPX, Direct Debit, DuitNow OBW & DuitNow QR payment channels.');
    }

    private function checkRequirements(): void
    {
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency set for this module');
        }
    }

    public function install(): bool
    {
        // Execute SQL installation
        if (!$this->installDatabase()) {
            return false;
        }

        // Create order state first
        $orderStateId = $this->createBayarcashOrderState();
        if (!$orderStateId) {
            return false;
        }

        return parent::install()
            && $this->registerHook(self::HOOKS)
            && $this->installConfiguration()
            && $this->installTabs();
    }

    private function createBayarcashOrderState(): int
    {
        $orderState = new OrderState();
        $orderState->name = array();
        foreach (Language::getLanguages() as $language) {
            $orderState->name[$language['id_lang']] = 'Awaiting Bayarcash Payment';
        }
        $orderState->send_email = false;
        $orderState->color = '#4169E1';
        $orderState->hidden = false;
        $orderState->delivery = false;
        $orderState->logable = false;
        $orderState->invoice = false;
        $orderState->paid = false;
        $orderState->module_name = $this->name;

        if (!$orderState->add()) {
            return 0;
        }

        // Save the order state ID in configuration
        Configuration::updateValue(self::CONFIG_ORDER_STATE, (int)$orderState->id);

        return (int)$orderState->id;
    }

    public function uninstall(): bool
    {
        return parent::uninstall()
            && $this->uninstallConfiguration()
            && $this->uninstallTabs();
    }

    private function getOrderStateInfo($stateId): array
    {
        $orderState = new OrderState($stateId, $this->context->language->id);
        return [
            'id' => (int)$stateId,
            'label' => pSQL($orderState->name)
        ];
    }

    private function installDatabase(): bool
    {
        $sql = [];
        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bayarcash_transactions` (
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `order_id` int(11) NOT NULL,
            `order_number` varchar(255) NOT NULL,
            `transaction_id` varchar(255) NOT NULL,
            `payment_channel` varchar(255) DEFAULT NULL,
            `order_state` int(11) NOT NULL,
            `order_state_label` varchar(255) DEFAULT NULL,
            `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `updated_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            PRIMARY KEY (`id`),
            KEY `idx_transaction_id` (`transaction_id`),
            KEY `idx_order_id` (`order_id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;';

        foreach ($sql as $query) {
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }
        return true;
    }

    public function getPortalApi(): BayarcashPortalApi
    {
        return $this->portalApi;
    }

    public function getContent(): string
    {
        Media::addJsDef([
            'bayarcashAjaxUrl' => $this->context->link->getAdminLink('AdminModules') . '&configure=' . $this->name . '&ajax=1'
        ]);

        // Handle AJAX request for portal refresh
        if (Tools::getValue('ajax') && Tools::getValue('action') === 'refreshPortals') {
            $pat = Tools::getValue('pat');
            if (empty($pat)) {
                die(json_encode([
                    'status' => 'error',
                    'message' => 'PAT is required'
                ]));
            }

            // Update the API instance with the new PAT
            $this->portalApi->setPat($pat);
            $result = $this->portalApi->getPortalList();
            die(json_encode($result));
        }

        $configForm = new BayarcashConfigForm($this);
        $output = '';

        // Check configuration status
        $pat = Configuration::get(self::CONFIG_PAT);
        $apiKey = Configuration::get(self::CONFIG_API_KEY);
        $enabled = Configuration::get(self::CONFIG_ENABLED);

        if ($enabled && (empty($pat) || empty($apiKey))) {
            $output .= $this->displayWarning(
                $this->l('Warning: Module is enabled but missing required configuration. Please provide both PAT and Portal selection.')
            );
        }

        if (Tools::isSubmit('submit' . $this->name)) {
            $output .= $configForm->processConfigurationUpdate();
        }

        return $output . $configForm->renderForm();
    }

    public function getTable()
    {
        return $this->table;
    }

    public function getIdentifier()
    {
        return $this->identifier;
    }

    public function getPath()
    {
        return $this->_path;
    }

    public function hookDisplayHeader($params): void
    {
        $this->context->controller->registerStylesheet(
            'modules-bc',
            'modules/' . $this->name . '/views/css/front.css',
            ['media' => 'all', 'priority' => 150]
        );
    }

    public function hookDisplayBackOfficeHeader(): string
    {
        if (Tools::getValue('configure') !== $this->name) {
            return '';
        }

        $this->context->controller->addJS([
            $this->_path . 'views/js/admin.js'
        ]);

        $this->context->controller->addCSS([
            $this->_path . 'views/css/admin.css'
        ]);

        Media::addJsDef([
            'bayarcashAjaxUrl' => $this->context->link->getAdminLink('AdminModules', true, [], [
                'configure' => $this->name,
                'ajax' => 1
            ]),
            'bayarcashToken' => Tools::getAdminTokenLite('AdminModules')
        ]);

        return '';
    }

    private function getChannelConfigurations(): array
    {
        return [
            1 => [
                'text' => $this->l('FPX Online Banking'),
                'logo' => 'fpx-1.png'
            ],
            4 => [
                'text' => $this->l('Credit Card'),
                'logo' => 'fpx-line-of-credit-1.png'
            ],
            5 => [
                'text' => $this->l('DuitNow Online Banking'),
                'logo' => 'dobw-1.png'
            ],
            6 => [
                'text' => $this->l('DuitNow QR'),
                'logo' => 'duitnow-qr-1.png'
            ],
            7 => [
                'text' => $this->l('SPayLater'),
                'logo' => 'spaylater-1.png'
            ],
            8 => [
                'text' => $this->l('Boost PayFlex'),
                'logo' => 'boost-payflex-1.png'
            ],
            9 => [
                'text' => $this->l('Indonesia Online Banking'),
                'logo' => 'qris-online-banking-1.png'
            ],
            10 => [
                'text' => $this->l('Indonesia e-Wallet'),
                'logo' => 'qris-online-banking-1.png'
            ],
            11 => [
                'text' => $this->l('NETS Singapore'),
                'logo' => 'logo-nets-colour.svg'
            ]
        ];
    }

    public function hookPaymentOptions($params): array
    {
        if (!$this->active ||
            !Configuration::get(self::CONFIG_ENABLED) ||
            !$this->checkCurrency($params['cart']) ||
            empty(Configuration::get(self::CONFIG_PAT)) ||
            empty(Configuration::get(self::CONFIG_API_KEY))) {
            return [];
        }

        $error = Tools::getValue('bayarcash_error');
        $paymentChannels = json_decode(Configuration::get(self::CONFIG_PAYMENT_CHANNELS), true) ?: [];
        $channelConfig = $this->getChannelConfigurations();
        $paymentOptions = [];

        foreach ($paymentChannels as $channelId) {
            if (isset($channelConfig[$channelId])) {
                $option = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
                $option->setCallToActionText($channelConfig[$channelId]['text'])
                    ->setAction($this->context->link->getModuleLink(
                        $this->name,
                        'validation',
                        ['channel' => $channelId],
                        true
                    ));

                if ($error) {
                    $this->context->smarty->assign(['bayarcash_error' => $error]);
                    $additionalInfo = $this->context->smarty->fetch('module:bayarcash/views/templates/hook/payment_error.tpl');
                    $option->setAdditionalInformation($additionalInfo);
                }

                $paymentOptions[] = $option;
            }
        }

        return $paymentOptions;
    }

    private function createPaymentOption($error): PrestaShop\PrestaShop\Core\Payment\PaymentOption
    {
        $paymentOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOption->setCallToActionText($this->l('Pay with Bayarcash'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', [], true))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/fpx-online-banking.png'));

        if ($error) {
            $this->context->smarty->assign(['bayarcash_error' => $error]);
            $additionalInfo = $this->context->smarty->fetch('module:bayarcash/views/templates/hook/payment_error.tpl');
            $paymentOption->setAdditionalInformation($additionalInfo);
        }

        return $paymentOption;
    }

    private function installConfiguration(): bool
    {
        $configs = [
            self::CONFIG_ENABLED => 0,
            self::CONFIG_PAT => '',
            self::CONFIG_API_KEY => '',
            self::CONFIG_SECRET_KEY => '',
            self::CONFIG_TEST_MODE => 0,
            self::CONFIG_PAYMENT_CHANNELS => '[]',
        ];

        foreach ($configs as $key => $value) {
            if (!Configuration::updateValue($key, $value)) {
                return false;
            }
        }

        return true;
    }

    private function uninstallConfiguration(): bool
    {
        $configs = [
            self::CONFIG_ENABLED,
            self::CONFIG_PAT,
            self::CONFIG_API_KEY,
            self::CONFIG_SECRET_KEY,
            self::CONFIG_TEST_MODE,
            self::CONFIG_PAYMENT_CHANNELS,
        ];

        foreach ($configs as $key) {
            if (!Configuration::deleteByName($key)) {
                return false;
            }
        }

        return true;
    }

    private function checkCurrency($cart): bool
    {
        $currencyOrder = new Currency($cart->id_currency);
        $currenciesModule = $this->getCurrency($cart->id_currency);

        if (empty($currenciesModule)) {
            return false;
        }

        foreach ($currenciesModule as $currencyModule) {
            if ($currencyOrder->id == $currencyModule['id_currency']) {
                return true;
            }
        }

        return false;
    }

    public function hookDisplayPaymentReturn($params): string
    {
        if (!$this->active || !isset($params['order']) || $params['order']->module !== $this->name) {
            return '';
        }

        $this->smarty->assign([
            'order' => $params['order'],
            'status' => $params['order']->getCurrentState(),
        ]);

        return $this->display(__FILE__, 'payment_return.tpl');
    }

    public function hookDisplayAdminOrderMainBottom($params): string
    {
        if (empty($params['id_order'])) {
            return '';
        }

        $order = new Order((int)$params['id_order']);
        if (!Validate::isLoadedObject($order) || $order->module !== $this->name) {
            return '';
        }

        // Get transaction information from database
        $transaction = Db::getInstance()->getRow('
            SELECT *
            FROM `' . _DB_PREFIX_ . 'bayarcash_transactions`
            WHERE order_number = "' . pSQL($order->reference) . '"
        ');

        // Get payment information
        $paymentInfo = '';
        if ($order->getOrderPaymentCollection()->count()) {
            $paymentInfo = $order->getOrderPaymentCollection()->getFirst()->transaction_id;
        }

        $channelConfig = $this->getChannelConfigurations();

        // Get channel information if available
        $channelInfo = null;
        if ($transaction && isset($transaction['payment_channel']) && isset($channelConfig[$transaction['payment_channel']])) {
            $channelInfo = $channelConfig[$transaction['payment_channel']];
        }

        $this->context->smarty->assign([
            'moduleName' => $this->name,
            'moduleDisplayName' => $this->displayName,
            'moduleLogoSrc' => $this->getPathUri() . 'views/img/bayarcash.png',
            'bayarcashOrder' => $order,
            'transactionId' => $paymentInfo,
            'orderId' => $params['id_order'],
            'channelConfig' => $channelConfig,
            'transaction' => $transaction,
            'channelInfo' => $channelInfo
        ]);

        return $this->context->smarty->fetch('module:bayarcash/views/templates/hook/displayAdminOrderMainBottom.tpl');
    }

    private function installTabs(): bool
    {
        $tab = new Tab();
        $tab->class_name = self::MODULE_ADMIN_CONTROLLER;
        $tab->module = $this->name;
        $tab->id_parent = -1;
        $tab->active = true;

        foreach (Language::getLanguages(false) as $lang) {
            $tab->name[$lang['id_lang']] = $this->displayName;
        }

        return $tab->add();
    }

    private function uninstallTabs(): bool
    {
        $idTab = Tab::getIdFromClassName(self::MODULE_ADMIN_CONTROLLER);
        if ($idTab) {
            $tab = new Tab($idTab);
            return $tab->delete();
        }
        return true;
    }
}