<?php
/**
 * Configuration form handler for Bayarcash module
 *
 * @author    Bayarcash Sdn Bhd
 * @copyright Bayarcash Sdn Bhd
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class BayarcashConfigForm
{
    private const PAYMENT_CHANNELS = [
        1 => 'Online Banking - FPX',
        4 => 'Credit Card',
        5 => 'Online Banking - DOBW',
        6 => 'DuitNow QR & e-Wallet',
        7 => 'SPayLater',
        8 => 'Boost PayFlex',
        9 => 'Indonesia Online Banking',
        10 => 'Indonesia e-Wallet',
        11 => 'NETS Singapore',
    ];

    private $module;
    private $context;

    public function __construct($module)
    {
        $this->module = $module;
        $this->context = Context::getContext();
    }

    public function renderForm(): string
    {
        $helper = new HelperForm();
        $this->setupFormHelper($helper);

        return $helper->generateForm([$this->getConfigFormFields()]);
    }

    public function processConfigurationUpdate(): string
    {
        $pat = trim(Tools::getValue(Bayarcash::CONFIG_PAT));
        $apiKey = trim(Tools::getValue(Bayarcash::CONFIG_API_KEY));
        $enabled = (bool)Tools::getValue(Bayarcash::CONFIG_ENABLED);
        $paymentChannels = Tools::getValue(Bayarcash::CONFIG_PAYMENT_CHANNELS);
        $testMode = (bool)Tools::getValue(Bayarcash::CONFIG_TEST_MODE);

        // Validate PAT and API Key if trying to enable the module
        if ($enabled && empty($pat)) {
            return $this->module->displayError($this->module->l('Personal Access Token (PAT) and Portal selection are required to enable the module.'));
        }

        // Validate payment channels
        if ($enabled && empty($paymentChannels)) {
            return $this->module->displayError($this->module->l('Please select at least one payment channel.'));
        }

        $configKeys = [
            Bayarcash::CONFIG_PAT => $pat,
            Bayarcash::CONFIG_API_KEY => $apiKey,
            Bayarcash::CONFIG_SECRET_KEY => trim(Tools::getValue(Bayarcash::CONFIG_SECRET_KEY)),
            Bayarcash::CONFIG_TEST_MODE => (string)$testMode,
            Bayarcash::CONFIG_PAYMENT_CHANNELS => json_encode($paymentChannels ?: []),
            Bayarcash::CONFIG_ENABLED => $enabled ? '1' : '0'
        ];

        foreach ($configKeys as $key => $value) {
            Configuration::updateValue($key, $value);
        }

        return $this->module->displayConfirmation($this->module->l('Settings updated'));
    }

    private function setupFormHelper(HelperForm $helper): void
    {
        $helper->show_toolbar = false;
        $helper->table = $this->module->getTable();
        $helper->module = $this->module;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->submit_action = 'submit' . $this->module->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->module->name
            . '&tab_module=' . $this->module->tab
            . '&module_name=' . $this->module->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = ['fields_value' => $this->getConfigFieldsValues()];
    }

    private function getConfigFormFields(): array
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->module->l('Bayarcash Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => $this->getConfigFormInputs(),
                'submit' => [
                    'title' => $this->module->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];
    }

    private function getConfigFormInputs(): array
    {
        $pat = Configuration::get(Bayarcash::CONFIG_PAT);
        $apiKey = Configuration::get(Bayarcash::CONFIG_API_KEY);
        $initialPortals = [];

        // Get initial portal list if PAT exists
        if (!empty($pat)) {
            $portalApi = $this->module->getPortalApi();
            $result = $portalApi->getPortalList();
            if ($result['status'] === 'success') {
                $initialPortals = $result['portals'];
            }
        }

        $inputs = [
            [
                'type' => 'switch',
                'label' => $this->module->l('Enable Bayarcash'),
                'name' => Bayarcash::CONFIG_ENABLED,
                'values' => $this->getSwitchValues($this->module->l('Enabled'), $this->module->l('Disabled')),
                'class' => 'bayarcash-switch',
            ],
            [
                'type' => 'textarea',
                'label' => $this->module->l('Personal Access Token (PAT)'),
                'name' => Bayarcash::CONFIG_PAT,
                'required' => true,
                'cols' => 60,
                'desc' => $this->module->l('Enter your Personal Access Token for authentication'),
                'class' => 'pat-input fixed-font',
            ],
            $this->getVerifyTokenButton(),
            $this->getPortalSelectionInput($initialPortals, $apiKey),
            [
                'type' => 'text',
                'label' => $this->module->l('Secret Key'),
                'name' => Bayarcash::CONFIG_SECRET_KEY,
                'required' => true,
                'class' => 'fixed-font',
            ],
            [
                'type' => 'select',
                'label' => $this->module->l('Payment Channels'),
                'name' => Bayarcash::CONFIG_PAYMENT_CHANNELS,
                'multiple' => true,
                'required' => true,
                'class' => 'chosen payment-channels-select',
                'options' => [
                    'query' => $this->getPaymentChannelOptions(),
                    'id' => 'id_option',
                    'name' => 'name'
                ],
                'desc' => $this->module->l('Select the payment channels you want to enable'),
            ],
            [
                'type' => 'switch',
                'label' => $this->module->l('Test Mode'),
                'name' => Bayarcash::CONFIG_TEST_MODE,
                'values' => $this->getSwitchValues($this->module->l('Yes'), $this->module->l('No')),
                'class' => 'bayarcash-switch',
            ]
        ];

        // Add corresponding CSS styles
        $this->context->controller->addCSS($this->module->getPathUri() . 'views/css/admin.css');

        // Add initial state information for JavaScript
        Media::addJsDef([
            'bayarcashInitialState' => [
                'hasToken' => !empty($pat),
                'portals' => $initialPortals,
                'selectedPortal' => $apiKey
            ]
        ]);

        return $inputs;
    }

    private function getVerifyTokenButton(): array
    {
        return [
            'type' => 'html',
            'name' => 'verify_token_button',
            'html_content' => '
            <div class="bayarcash-verify-token">
                <div class="col-lg-9 col-lg-offset-3">
                    <div class="verify-token-wrapper">
                        <button type="button" id="verify_token" class="btn btn-primary">
                            <i class="icon icon-check"></i> ' . $this->module->l('Verify Token') . '
                        </button>
                        <div id="token_verification_result" class="help-block verification-result" style="display:none;"></div>
                    </div>
                </div>
            </div>'
        ];
    }

    private function getPortalSelectionInput($initialPortals, $apiKey): array
    {
        return [
            'type' => 'html',
            'name' => 'portal_selection_container',
            'html_content' => '
            <div id="portal_selection_wrapper" class="portal bayarcash-portal-selection">
                <label class="control-label col-lg-3 required">
                    ' . $this->module->l('Portal Key') . '
                </label>
                <div class="col-lg-9">
                    <div class="portal-select-container">
                        <select name="' . Bayarcash::CONFIG_API_KEY . '" id="portal_selection" class="form-control">
                            <option value="">' . $this->module->l('Select a portal') . '</option>'
                . $this->generatePortalOptions($initialPortals, $apiKey) . '
                        </select>
                        <p class="help-block">' . $this->module->l('Select your Bayarcash portal') . '</p>
                    </div>
                </div>
            </div>'
        ];
    }

    private function getPaymentChannelOptions(): array
    {
        return array_map(function($id, $name) {
            return [
                'id_option' => $id,
                'name' => $name
            ];
        }, array_keys(self::PAYMENT_CHANNELS), array_values(self::PAYMENT_CHANNELS));
    }

    private function generatePortalOptions($portals, $selectedPortal): string
    {
        $html = '';
        foreach ($portals as $portal) {
            $selected = ($portal['id'] === $selectedPortal) ? 'selected="selected"' : '';
            $html .= '<option value="' . htmlspecialchars($portal['id']) . '" ' . $selected . '>'
                . htmlspecialchars($portal['name'])
                . '</option>';
        }
        return $html;
    }

    private function getSwitchValues(string $onLabel, string $offLabel): array
    {
        return [
            [
                'id' => 'active_on',
                'value' => 1,
                'label' => $onLabel,
            ],
            [
                'id' => 'active_off',
                'value' => 0,
                'label' => $offLabel,
            ],
        ];
    }

    private function getConfigFieldsValues(): array
    {
        $configKeys = [
            Bayarcash::CONFIG_PAT,
            Bayarcash::CONFIG_ENABLED,
            Bayarcash::CONFIG_API_KEY,
            Bayarcash::CONFIG_SECRET_KEY,
            Bayarcash::CONFIG_TEST_MODE,
            Bayarcash::CONFIG_PAYMENT_CHANNELS,
        ];

        $values = [];
        foreach ($configKeys as $key) {
            $values[$key] = Configuration::get($key);
        }

        // Handle payment channels separately
        $paymentChannels = json_decode(Configuration::get(Bayarcash::CONFIG_PAYMENT_CHANNELS), true) ?: [];
        $values[Bayarcash::CONFIG_PAYMENT_CHANNELS . '[]'] = $paymentChannels;

        return $values;
    }
}