<?php
/**
 * Bayarcash Portal API handling class
 *
 * @author    Bayarcash Sdn Bhd
 * @copyright Bayarcash Sdn Bhd
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace admin;
if (!defined('_PS_VERSION_')) {
    exit;
}

class BayarcashPortalApi
{
    // API endpoints
    private const API_ENDPOINT_SANDBOX = 'https://console.bayarcash-sandbox.com/api/v2/portals';
    private const API_ENDPOINT_PRODUCTION = 'https://console.bayar.cash/api/v2/portals';

    private $testMode;
    private $pat;

    public function __construct(bool $testMode = false, string $pat = '')
    {
        $this->testMode = $testMode;
        $this->pat = $pat;
    }

    /**
     * Set Personal Access Token (PAT)
     *
     * @param string $pat Personal Access Token
     * @return void
     */
    public function setPat(string $pat): void
    {
        $this->pat = $pat;
    }

    /**
     * Set test mode
     *
     * @param bool $testMode Test mode status
     * @return void
     */
    public function setTestMode(bool $testMode): void
    {
        $this->testMode = $testMode;
    }

    /**
     * Get current API endpoint based on test mode
     *
     * @return string API endpoint URL
     */
    private function getApiEndpoint(): string
    {
        return $this->testMode ? self::API_ENDPOINT_SANDBOX : self::API_ENDPOINT_PRODUCTION;
    }

    /**
     * Get list of available portals
     *
     * @param string|null $pat Optional PAT to override the instance PAT
     * @return array Response containing status and portal data or error information
     */
    public function getPortalList(?string $pat = null): array
    {
        // Use passed PAT if available, otherwise use stored PAT
        $pat = $pat ?: $this->pat;
        if (empty($pat)) {
            return [
                'error' => 'PAT is empty',
                'status' => 'error'
            ];
        }

        $curl = curl_init();
        $apiEndpoint = $this->getApiEndpoint();

        curl_setopt_array($curl, [
            CURLOPT_URL => $apiEndpoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_HTTPHEADER => [
                "Authorization: Bearer " . $pat,
                "Accept: application/json",
                "Content-Type: application/json"
            ],
        ]);

        $response = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $err = curl_error($curl);

        // Debug information
        $debug = [
            'endpoint' => $apiEndpoint,
            'httpCode' => $httpCode,
            'curlError' => $err,
            'response' => $response
        ];

        curl_close($curl);

        if ($err) {
            return [
                'status' => 'error',
                'message' => 'Curl error: ' . $err,
                'debug' => $debug
            ];
        }

        $data = json_decode($response, true);

        if (json_last_error() !== JSON_ERROR_NONE) {
            return [
                'status' => 'error',
                'message' => 'JSON decode error: ' . json_last_error_msg(),
                'debug' => $debug
            ];
        }

        if (!isset($data['data'])) {
            return [
                'status' => 'error',
                'message' => 'Invalid response format',
                'debug' => $debug
            ];
        }

        $portals = [];
        foreach ($data['data'] as $portal) {
            $portals[] = [
                'id' => $portal['portal_key'],
                'name' => $portal['portal_name']
            ];
        }

        return [
            'status' => 'success',
            'portals' => $portals,
            'debug' => $debug
        ];
    }

    /**
     * Verify if PAT is valid by attempting to fetch portals
     *
     * @param string $pat PAT to verify
     * @return bool True if PAT is valid, false otherwise
     */
    public function verifyPat(string $pat): bool
    {
        $result = $this->getPortalList($pat);
        return $result['status'] === 'success';
    }
}