<?php
/**
 * Handle payment callbacks from Bayarcash payment gateway.
 *
 * @author    Bayarcash Sdn Bhd
 * @copyright Bayarcash Sdn Bhd
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

class BayarcashCallbackModuleFrontController extends ModuleFrontController
{
    /**
     * @var Order
     */
    private $order;

    public function init()
    {
        parent::init();

        // Get order ID and secure key from URL parameters
        $orderId = (int)Tools::getValue('order_id');
        $secureKey = Tools::getValue('secure_key');

        if ($orderId) {
            // Get order by ID
            $this->order = new Order($orderId);

            if (Validate::isLoadedObject($this->order)) {
                // Verify secure key
                $customer = new Customer($this->order->id_customer);
                if (Validate::isLoadedObject($customer) && $customer->secure_key === $secureKey) {
                    // Force customer authentication
                    $this->context->updateCustomer($customer);

                    // Create or update cart with customer info
                    if (!$this->context->cart->id) {
                        $this->context->cart = new Cart();
                        $this->context->cart->id_customer = (int)$customer->id;
                        $this->context->cart->id_lang = (int)$this->context->language->id;
                        $this->context->cart->id_currency = (int)$this->order->id_currency;
                        $this->context->cart->id_shop = (int)$this->context->shop->id;
                        if (!$this->context->cart->add()) {
                            throw new Exception('Could not create new cart');
                        }
                    } else {
                        $this->context->cart->id_customer = (int)$customer->id;
                        $this->context->cart->id_currency = (int)$this->order->id_currency;
                    }
                    $this->context->cart->update();

                    // Ensure customer is logged in
                    if (!$this->context->customer->isLogged()) {
                        $this->context->customer = $customer;
                        $this->context->cookie->id_customer = (int)$customer->id;
                        $this->context->cookie->customer_lastname = $customer->lastname;
                        $this->context->cookie->customer_firstname = $customer->firstname;
                        $this->context->cookie->logged = 1;
                        $this->context->cookie->is_guest = $customer->is_guest;
                        $this->context->cookie->passwd = $customer->passwd;
                        $this->context->cookie->email = $customer->email;
                        $this->context->cookie->write();
                    }
                }
            }
        }
    }

    public function postProcess(): void
    {
        try {
            // Log raw POST data
            PrestaShopLogger::addLog(
                'Bayarcash callback received: ' . http_build_query($_POST),
                1,
                null,
                'Bayarcash',
                null,
                true
            );

            // Validate required fields
            $this->validatePayload($_POST);

            // Process based on record type
            switch ($_POST['record_type']) {
                case 'pre_transaction':
                    $this->handlePreTransaction($_POST);
                    break;
                case 'transaction':
                case 'transaction_receipt':
                    $this->handleCompletedTransaction($_POST);
                    break;
                default:
                    throw new Exception('Invalid record_type received');
            }

            // Log success response
            PrestaShopLogger::addLog(
                'Bayarcash callback processed successfully for transaction: ' . $_POST['transaction_id'],
                1,
                null,
                'Bayarcash',
                null,
                true
            );

            // Get order from class property
            if (!Validate::isLoadedObject($this->order)) {
                throw new Exception('Order not found.');
            }

            // Get customer for secure key
            $customer = new Customer($this->order->id_customer);
            if (!Validate::isLoadedObject($customer)) {
                throw new Exception('Customer not found.');
            }

            // Redirect to order confirmation page with all necessary parameters
            Tools::redirect('index.php?controller=order-confirmation' .
                '&id_cart=' . $this->order->id_cart .
                '&id_module=' . $this->module->id .
                '&id_order=' . $this->order->id .
                '&key=' . $customer->secure_key);

        } catch (Exception $e) {
            PrestaShopLogger::addLog(
                'Bayarcash callback error: ' . $e->getMessage(),
                3,
                null,
                'Bayarcash',
                null,
                true
            );

            // Redirect to order history page in case of error
            Tools::redirect('index.php?controller=history');
        }
    }

    private function validatePayload(array $payload): void
    {
        $requiredFields = ['record_type', 'transaction_id', 'order_number'];

        foreach ($requiredFields as $field) {
            if (!isset($payload[$field])) {
                throw new Exception("Missing required field: {$field}");
            }
        }
    }

    private function getFailedPaymentOrderState(): int
    {
        // Try to get existing failed payment state ID
        $failedStateId = Configuration::get('BAYARCASH_OS_FAILED');

        // Validate the state exists and is valid
        if ($failedStateId && OrderState::existsInDatabase($failedStateId, Db::getInstance())) {
            return (int)$failedStateId;
        }

        // Create new order state if it doesn't exist
        $orderState = new OrderState();
        $orderState->name = array();
        foreach (Language::getLanguages() as $language) {
            $orderState->name[$language['id_lang']] = 'Bayarcash Failed Payment';
        }
        $orderState->send_email = false;
        $orderState->color = '#e74c3c'; // Red color
        $orderState->hidden = false;
        $orderState->delivery = false;
        $orderState->logable = true;
        $orderState->invoice = false;
        $orderState->paid = false;
        $orderState->module_name = 'bayarcash';

        if ($orderState->add()) {
            $failedStateId = $orderState->id;
            Configuration::updateValue('BAYARCASH_OS_FAILED', (int)$failedStateId);
            return (int)$failedStateId;
        }

        // Fallback to default error state if all else fails
        return (int)Configuration::get('PS_OS_ERROR');
    }

    private function handlePreTransaction(array $payload): void
    {
        // Get order from class property
        if (!Validate::isLoadedObject($this->order)) {
            throw new Exception('Order not found for reference: ' . $payload['order_number']);
        }

        try {
            // Update only transaction_id in bayarcash_transactions
            $existingTransaction = Db::getInstance()->getRow(
                'SELECT * FROM `' . _DB_PREFIX_ . 'bayarcash_transactions` 
            WHERE order_number = "' . pSQL($this->order->reference) . '"'
            );

            if ($existingTransaction) {
                // Update only transaction_id
                Db::getInstance()->update(
                    'bayarcash_transactions',
                    ['transaction_id' => pSQL($payload['transaction_id'])],
                    'order_number = "' . pSQL($this->order->reference) . '"'
                );

                PrestaShopLogger::addLog(
                    sprintf(
                        'Updated Bayarcash transaction_id for order %s: %s',
                        $this->order->reference,
                        $payload['transaction_id']
                    ),
                    1,
                    null,
                    'Order',
                    $this->order->id,
                    true
                );

                // Add order note for transaction initiation
                $orderNote = sprintf(
                    'Bayarcash payment initiated. Transaction ID: %s, Exchange Reference: %s',
                    $payload['transaction_id'],
                    $payload['exchange_reference_number']
                );

                $this->addOrderNote($this->order, $orderNote);
            }
        } catch (Exception $e) {
            PrestaShopLogger::addLog(
                'Error updating Bayarcash transaction: ' . $e->getMessage(),
                3,
                null,
                'Order',
                $this->order->id,
                true
            );
        }
    }

    private function handleCompletedTransaction(array $payload): void
    {
        // Get order from class property
        if (!Validate::isLoadedObject($this->order)) {
            throw new Exception('Order not found for reference: ' . $payload['order_number']);
        }

        // Determine target order state based on payment status
        $targetState = $payload['status'] == 3
            ? Configuration::get('PS_OS_PAYMENT')
            : $this->getFailedPaymentOrderState();

        // Check if the current state is different from the target state
        if ($this->order->current_state != $targetState) {
            // Get order state information
            $orderState = new OrderState($targetState, $this->context->language->id);

            try {
                // Update transaction data with specific fields
                $existingTransaction = Db::getInstance()->getRow(
                    'SELECT * FROM `' . _DB_PREFIX_ . 'bayarcash_transactions` 
                WHERE order_number = "' . pSQL($this->order->reference) . '"'
                );

                if ($existingTransaction) {
                    // Update only transaction_id, order_state and order_state_label
                    Db::getInstance()->update(
                        'bayarcash_transactions',
                        [
                            'transaction_id' => pSQL($payload['transaction_id']),
                            'order_state' => (int)$targetState,
                            'order_state_label' => pSQL($orderState->name)
                        ],
                        'order_number = "' . pSQL($this->order->reference) . '"'
                    );

                    PrestaShopLogger::addLog(
                        sprintf(
                            'Updated Bayarcash transaction state for order %s: %s',
                            $this->order->reference,
                            $orderState->name
                        ),
                        1,
                        null,
                        'Order',
                        $this->order->id,
                        true
                    );
                }
            } catch (Exception $e) {
                PrestaShopLogger::addLog(
                    'Error updating Bayarcash transaction: ' . $e->getMessage(),
                    3,
                    null,
                    'Order',
                    $this->order->id,
                    true
                );
            }

            // Add transaction details to order note before status change
            $orderNote = sprintf(
                'Payment %s. Transaction ID: %s, Exchange Reference: %s, Bank: %s, Amount: %s %s, Status: %s (%s), DateTime: %s',
                $payload['status'] == 3 ? 'completed' : 'failed',
                $payload['transaction_id'],
                $payload['exchange_reference_number'],
                $payload['payer_bank_name'],
                $payload['currency'],
                $payload['amount'],
                $payload['status'],
                $payload['status_description'],
                urldecode($payload['datetime'])
            );

            $this->addOrderNote($this->order, $orderNote);

            // Update order status
            $this->order->setCurrentState($targetState);

            // Update payment transaction ID if not already set
            $payment = $this->order->getOrderPayments()[0];
            if ($payment && !$payment->transaction_id) {
                $payment->transaction_id = $payload['transaction_id'];
                $payment->save();
            }
        } else {
            // Log that status remains unchanged for PrestaShop order
            $currentOrderState = new OrderState($this->order->current_state, $this->context->language->id);
            PrestaShopLogger::addLog(
                sprintf(
                    'Order status unchanged for %s - Current state: %s (%d), Received status: %s, Transaction ID: %s',
                    $this->order->reference,
                    $currentOrderState->name,
                    $this->order->current_state,
                    $payload['status_description'],
                    $payload['transaction_id']
                ),
                1,
                null,
                'Order',
                $this->order->id,
                true
            );

            // Check and log Bayarcash transaction table status
            try {
                $existingTransaction = Db::getInstance()->getRow(
                    'SELECT * FROM `' . _DB_PREFIX_ . 'bayarcash_transactions` 
                WHERE order_number = "' . pSQL($this->order->reference) . '"'
                );

                if ($existingTransaction) {
                    PrestaShopLogger::addLog(
                        sprintf(
                            'Bayarcash transaction status unchanged for order %s - Current state: %s (%d), Received transaction ID: %s',
                            $this->order->reference,
                            $existingTransaction['order_state_label'],
                            $existingTransaction['order_state'],
                            $payload['transaction_id']
                        ),
                        1,
                        null,
                        'Order',
                        $this->order->id,
                        true
                    );
                } else {
                    PrestaShopLogger::addLog(
                        sprintf(
                            'No Bayarcash transaction record found for order %s with transaction ID: %s',
                            $this->order->reference,
                            $payload['transaction_id']
                        ),
                        2, // Warning level
                        null,
                        'Order',
                        $this->order->id,
                        true
                    );
                }
            } catch (Exception $e) {
                PrestaShopLogger::addLog(
                    sprintf(
                        'Error checking Bayarcash transaction record: %s (Order: %s)',
                        $e->getMessage(),
                        $this->order->reference
                    ),
                    3,
                    null,
                    'Order',
                    $this->order->id,
                    true
                );
            }
        }
    }

    private function addOrderNote(Order $order, string $message): void
    {
        // Get customer email
        $customer = new Customer($order->id_customer);
        $customerEmail = $customer->email;

        // Get or create customer thread
        $customerThreadId = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($customerEmail, $order->id);

        if (!$customerThreadId) {
            $customerThread = new CustomerThread();
            $customerThread->id_shop = $order->id_shop;
            $customerThread->id_lang = $order->id_lang;
            $customerThread->id_contact = 0;
            $customerThread->id_customer = $order->id_customer;
            $customerThread->id_order = $order->id;
            $customerThread->status = 'open';
            $customerThread->email = $customerEmail;
            $customerThread->token = Tools::passwdGen(12);
            $customerThread->add();
            $customerThreadId = $customerThread->id;
        }

        $customerMessage = new CustomerMessage();
        $customerMessage->id_customer_thread = $customerThreadId;
        $customerMessage->id_employee = 0;
        $customerMessage->message = $message;
        $customerMessage->private = 1;
        $customerMessage->add();

        // Log order note creation
        PrestaShopLogger::addLog(
            sprintf(
                'Bayarcash order note added for Order %d: %s',
                $order->id,
                $message
            ),
            1,
            null,
            'Order',
            $order->id,
            true
        );
    }
}