<?php
/**
 * Accept payment from Malaysia. Bayarcash support FPX, Direct Debit, DuitNow OBW & DuitNow QR payment channels.
 *
 * @author    Bayarcash Sdn Bhd
 * @copyright Bayarcash Sdn Bhd
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

class BayarcashValidationModuleFrontController extends ModuleFrontController
{
    private string $baseUrl;
    private string|false $pat;
    private string|false $portalKey;

    public function __construct()
    {
        parent::__construct();
        $testMode = (bool)Configuration::get('BAYARCASH_TEST_MODE');
        $this->baseUrl = $testMode
            ? 'https://console.bayarcash-sandbox.com/api/v2'
            : 'https://console.bayar.cash/api/v2';
        $this->pat = Configuration::get('BAYARCASH_PAT');
        $this->portalKey = Configuration::get('BAYARCASH_API_KEY');
    }

    private function getSelectedPaymentChannel()
    {
        // Get the channel from URL parameter
        $channel = (int)Tools::getValue('channel');

        // Validate if the channel is in the allowed payment channels
        $allowedChannels = json_decode(Configuration::get('BAYARCASH_PAYMENT_CHANNELS'), true) ?: [];

        if (!in_array($channel, $allowedChannels)) {
            throw new Exception('Invalid payment channel selected');
        }

        return $channel;
    }

    private function getBayarcashOrderState(): int
    {
        // Get the Bayarcash order state
        $bayarcashStateId = Configuration::get('BAYARCASH_ORDER_STATE');

        if (!$bayarcashStateId || !OrderState::existsInDatabase($bayarcashStateId, Db::getInstance())) {
            PrestaShopLogger::addLog(
                'Bayarcash order state not found. Using default payment state.',
                3,
                null,
                'Bayarcash',
                0,
                true
            );
            return (int)Configuration::get('PS_OS_PAYMENT');
        }

        return (int)$bayarcashStateId;
    }

    private function createPaymentIntent($orderData)
    {
        $paymentChannel = $this->getSelectedPaymentChannel();

        $payload = [
            'payment_channel' => $paymentChannel,
            'portal_key' => $this->portalKey,
            'order_number' => $orderData['order_number'],
            'payer_telephone_number' => $orderData['phone'] ?? '',
            'return_url' => $orderData['return_url'],
            'amount' => $orderData['amount'],
            'payer_name' => $orderData['customer_name'],
            'payer_email' => $orderData['customer_email']
        ];

        $ch = curl_init($this->baseUrl . '/payment-intents');

        curl_setopt_array($ch, [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_HTTPHEADER => [
                'Accept: application/json',
                'Content-Type: application/json',
                'Authorization: Bearer ' . $this->pat
            ],
            CURLOPT_POSTFIELDS => json_encode($payload),
            CURLOPT_SSL_VERIFYPEER => true
        ]);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ($response === false) {
            $error = curl_error($ch);
            curl_close($ch);
            throw new Exception('cURL Error: ' . $error);
        }

        curl_close($ch);

        if ($httpCode !== 201) {
            throw new Exception('API Error: HTTP ' . $httpCode . ' - ' . $response);
        }

        $result = json_decode($response, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new Exception('Invalid JSON response from API');
        }

        return $result;
    }

    private function getCustomerPhone($customer)
    {
        $address = new Address($this->context->cart->id_address_invoice);
        return $address->phone_mobile ?: $address->phone;
    }

    public function postProcess(): void
    {
        if (!$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $cart = $this->context->cart;

        // Validate cart and customer
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        try {
            $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

            // Get Bayarcash order state
            $orderStateId = $this->getBayarcashOrderState();

            // Create PrestaShop order first
            $this->module->validateOrder(
                (int)$cart->id,
                $orderStateId,
                $total,
                $this->module->displayName,
                null,
                [],
                (int)$cart->id_currency,
                false,
                $customer->secure_key
            );

            // Get the created order ID
            $orderId = Order::getOrderByCartId($cart->id);

            if (!$orderId) {
                throw new Exception('Failed to create order');
            }

            // Get the order object
            $order = new Order($orderId);

            // Prepare order data with order ID instead of cart ID
            $orderData = [
                'order_number' => $order->reference,
                'amount' => $total,
                'customer_name' => $customer->firstname . ' ' . $customer->lastname,
                'customer_email' => $customer->email,
                'phone' => $this->getCustomerPhone($customer),
                'return_url' => $this->context->link->getModuleLink(
                    $this->module->name,
                    'callback',
                    [
                        'order_id' => $orderId,
                        'secure_key' => $customer->secure_key
                    ],
                    true
                )
            ];

            $this->logOrderData($orderData, $orderId);

            // Create payment intent
            $response = $this->createPaymentIntent($orderData);

            if (!isset($response['url'])) {
                throw new Exception('Invalid response from Bayarcash API');
            }

            // Save transaction data
            $paymentChannel = $this->getSelectedPaymentChannel();
            $this->saveTransactionData($orderId, $order->reference, $response, $paymentChannel);

            // Update order with transaction information
            $order->setCurrentState($orderStateId);
            $order->save();

            // Add order payment record
            if (isset($response['id'])) {
                $payment = new OrderPayment();
                $payment->order_reference = $order->reference;
                $payment->id_currency = $order->id_currency;
                $payment->amount = $total;
                $payment->payment_method = $this->module->displayName;
                $payment->transaction_id = $response['id'];
                $payment->add();
            }

            // Redirect to Bayarcash payment page
            Tools::redirect($response['url']);

        } catch (Exception $e) {
            PrestaShopLogger::addLog(
                'Bayarcash payment error: ' . $e->getMessage(),
                3,
                null,
                'Order',
                $orderId ?? 0,
                true
            );

            // Store error message and redirect to payment step
            $errorMessage = $this->trans(
                'We encountered an issue processing your payment. Please try again or select a different payment method.',
                [],
                'Modules.Bayarcash.Shop'
            );

            Tools::redirect('index.php?controller=order&step=payment&bayarcash_error='.urlencode($errorMessage));
        }
    }

    private function getOrderStateInfo($stateId): array
    {
        $orderState = new OrderState($stateId, $this->context->language->id);
        return [
            'id' => (int)$stateId,
            'label' => pSQL($orderState->name)
        ];
    }

    private function saveTransactionData($orderId, $orderReference, $response, $paymentChannel): bool
    {
        try {
            // Get order state information
            $orderState = $this->getOrderStateInfo($this->getBayarcashOrderState());

            // Prepare the data for insertion
            $data = [
                'order_id' => (int)$orderId,
                'order_number' => pSQL($orderReference),
                'transaction_id' => pSQL($response['id']),
                'payment_channel' => pSQL($paymentChannel),
                'order_state' => (int)$orderState['id'],
                'order_state_label' => $orderState['label']
            ];

            // Insert into the bayarcash_transactions table
            $result = Db::getInstance()->insert(
                'bayarcash_transactions',
                $data
            );

            if (!$result) {
                PrestaShopLogger::addLog(
                    sprintf(
                        'Failed to save Bayarcash transaction data for order %s (State: %s - %s)',
                        $orderId,
                        $orderState['id'],
                        $orderState['label']
                    ),
                    3, // Error level
                    null,
                    'Order',
                    $orderId,
                    true
                );
                return false;
            }

            return true;
        } catch (Exception $e) {
            PrestaShopLogger::addLog(
                sprintf(
                    'Error saving Bayarcash transaction: %s (Order: %s, State: %s - %s)',
                    $e->getMessage(),
                    $orderId,
                    $orderState['id'],
                    $orderState['label']
                ),
                3,
                null,
                'Order',
                $orderId,
                true
            );
            return false;
        }
    }

    private function logOrderData(array $orderData, int $orderId): void
    {
        // Format the data for logging
        $logMessage = sprintf(
            'Bayarcash payment initiated - Order Data: %s',
            json_encode($orderData, JSON_PRETTY_PRINT)
        );

        // Log using PrestaShop's native logger
        PrestaShopLogger::addLog(
            $logMessage,
            1, // Info level
            null,
            'Order',
            $orderId,
            true
        );
    }
}